@@ -1,4 +1,4 @@
-/* $Id: geoid.c 4257 2007-01-18 03:56:43Z ckuethe $ */
+/* $Id: geoid.c 4958 2009-01-08 17:45:01Z ckuethe $ */
 /* 
  * geoid.c -- ECEF to WGS84 conversions, including ellipsoid-to-MSL height
  *
@@ -117,7 +117,7 @@ void ecef_to_wgs84fix(struct gps_data_t *gpsdata,
     heading = atan2(fix_minuz(veast), fix_minuz(vnorth));
     /*@ +evalorder @*/
     if (heading < 0)
-	heading += 2 * PI;
+	heading += 2 * GPS_PI;
     gpsdata->fix.track = heading * RAD_2_DEG;
 }
 
@@ -135,38 +135,3 @@ static double fix_minuz(double d)
 {
     return ((d == 0.0) ? 0.0 : d);
 }
-
-#ifdef TESTMAIN
-#include <stdio.h>
-#include <stdlib.h>
-
-int main(int argc, char **argv)
-{
-    double	lat, lon;
-
-    if (argc != 3) {
-	fprintf(stderr,"Usage: %s lat lon\n",argv[0]);
-	return 1;
-    }	
-
-    lat=atof(argv[1]);
-    lon=atof(argv[2]);
-	
-    if (lat > 90. || lat < -90.)
-    {
-	fprintf(stderr," -90 <= lat=%s(%.f) <= 90 ?\n",argv[1],lat);
-	return 1;
-    }	
-
-    if (lon > 180. || lat < -180.)
-    {
-	fprintf(stderr," -180 <= lon=%s(%.f) <= 180 ?\n",argv[2],lon);
-	return 1;
-    }
-	
-    printf(" lat= %f lon= %f geoid correction= %f\n",
-	   lat, lon, wgs84_separation(lat, lon));
-
-    return 0;
-}
-#endif /* TESTMAIN */
