@@ -63,10 +63,10 @@ static gps_mask_t get_packet(struct gps_device_t *session)
 static void onsig(int sig)
 {
     if (sig == SIGALRM) {
-	(void)fputs("gpsctl: packet recognition timed out.\n", stdout);
+	gpsd_report(LOG_ERROR, "packet recognition timed out.\n");
 	exit(1);
     } else {
-	(void)printf("gpsctl: killed by signal %d\n", sig);
+	gpsd_report(LOG_ERROR, "killed by signal %d\n", sig);
 	exit(0);
     }
 }
@@ -74,23 +74,34 @@ static void onsig(int sig)
 int main(int argc, char **argv)
 {
     int option, status;
-    char *err_str, *device = NULL, *speed = NULL, *devtype = NULL, *control = NULL;
+    char *err_str, *device = NULL, *devtype = NULL; 
+    char *speed = NULL, *control = NULL, *rate = NULL;
     bool to_binary = false, to_nmea = false, reset = false; 
     bool lowlevel=false, echo=false;
     struct gps_data_t *gpsdata = NULL;
-    struct gps_type_t *forcetype = NULL;
-    struct gps_type_t **dp;
+    const struct gps_type_t *forcetype = NULL;
+    const struct gps_type_t **dp;
+    unsigned int timeout = 4;
+#ifdef ALLOW_CONTROLSEND
     char cooked[BUFSIZ];
     ssize_t cooklen = 0;
-    int timeout = 4;
+#endif /* ALLOW_RECONFIGURE */
 
-#define USAGE	"usage: gpsctl [-l] [-b | -n | -r] [-D n] [-s speed] [-T timeout] [-V] [-t devtype] [-c control] [-e] <device>\n"
-    while ((option = getopt(argc, argv, "bc:efhlnrs:t:D:T:V")) != -1) {
+#define USAGE	"usage: gpsctl [-l] [-b | -n | -r] [-D n] [-s speed] [-c rate] [-T timeout] [-V] [-t devtype] [-x control] [-e] <device>\n"
+    while ((option = getopt(argc, argv, "befhlnrs:t:x:D:T:V")) != -1) {
 	switch (option) {
 	case 'b':		/* switch to vendor binary mode */
 	    to_binary = true;
 	    break;
-	case 'c':		/* ship specified control string */
+	case 'c':
+#ifdef ALLOW_RECONFIGURE
+	    rate = optarg;
+#else
+	    gpsd_report(LOG_ERROR, "cycle-change capability has been conditioned out.\n");
+#endif /* ALLOW_RECONFIGURE */
+	    break;
+	case 'x':		/* ship specified control string */
+#ifdef ALLOW_CONTROLSEND
 	    control = optarg;
 	    lowlevel = true;
 	    if ((cooklen = hex_escapes(cooked, control)) <= 0) {
@@ -98,6 +109,9 @@ int main(int argc, char **argv)
 			    "invalid escape string (error %d)\n", (int)cooklen);
 		exit(1);
 	    }
+#else
+	    gpsd_report(LOG_ERROR, "control_send capability has been conditioned out.\n");	    
+#endif /* ALLOW_CONTROLSEND */
 	    break;
 	case 'e':		/* echo specified control string with wrapper */
 	    lowlevel = true;
@@ -108,6 +122,7 @@ int main(int argc, char **argv)
 	    break;
         case 'l':		/* list known device types */
 	    for (dp = gpsd_drivers; *dp; dp++) {
+#ifdef ALLOW_RECONFIGURE
 		if ((*dp)->mode_switcher != NULL)
 		    (void)fputs("-[bn]\t", stdout);
 		else
@@ -116,35 +131,54 @@ int main(int argc, char **argv)
 		    (void)fputs("-s\t", stdout);
 		else
 		    (void)fputc('\t', stdout);
-		if ((*dp)->control_send != NULL || strcmp((*dp)->type_name, "uBlox UBX binary")==0)
+		if ((*dp)->rate_switcher != NULL)
 		    (void)fputs("-c\t", stdout);
 		else
 		    (void)fputc('\t', stdout);
+#endif /* ALLOW_RECONFIGURE */
+#ifdef ALLOW_CONTROLSEND
+		if ((*dp)->control_send != NULL)
+		    (void)fputs("-x\t", stdout);
+		else
+		    (void)fputc('\t', stdout);
+#endif /* ALLOW_CONTROLSEND */
 		(void)puts((*dp)->type_name);
 	    }
 	    exit(0);
 	case 'n':		/* switch to NMEA mode */
+#ifdef ALLOW_RECONFIGURE
 	    to_nmea = true;
+#else
+	    gpsd_report(LOG_ERROR, "speed-change capability has been conditioned out.\n");
+#endif /* ALLOW_RECONFIGURE */
 	    break;
 	case 'r':		/* force-switch to default mode */
+#ifdef ALLOW_RECONFIGURE
 	    reset = true;
 	    lowlevel = false;	/* so we'll abort if the daemon is running */
+#else
+	    gpsd_report(LOG_ERROR, "reset capability has been conditioned out.\n");
+#endif /* ALLOW_RECONFIGURE */
 	    break;
 	case 's':		/* change output baud rate */
+#ifdef ALLOW_RECONFIGURE
 	    speed = optarg;
+#else
+	    gpsd_report(LOG_ERROR, "speed-change capability has been conditioned out.\n");
+#endif /* ALLOW_RECONFIGURE */
 	    break;
 	case 't':		/* force the device type */
 	    devtype = optarg;
 	    break;
 	case 'T':		/* force the device type */
-	    timeout = atoi(optarg);
+	    timeout = (unsigned)atoi(optarg);
 	    break;
 	case 'D':		/* set debugging level */
 	    debuglevel = atoi(optarg);
 	    gpsd_hexdump_level = debuglevel;
 	    break;
 	case 'V':
-	    (void)fprintf(stderr, "gpsctl at svn revision $Rev: 5135 $\n");
+	    (void)fprintf(stderr, "gpsctl at svn revision $Rev$\n");
 	    break;
 	case 'h':
 	default:
@@ -174,10 +208,10 @@ int main(int argc, char **argv)
 	    gpsd_report(LOG_ERROR, "%d driver type names match '%s'.\n",
 			matchcount, devtype);
 	}
-    }
+    }    
 
     if ((int)to_nmea + (int)to_binary + (int)reset > 1) {
-	(void)fprintf(stderr, "gpsctl: make up your mind, would you?\n");
+	gpsd_report(LOG_ERROR, "make up your mind, would you?\n");
 	exit(0);
     }
 
@@ -198,8 +232,7 @@ int main(int argc, char **argv)
 	    case NL_NOCONNECT: err_str ="can't connect"; break;
 	    default:           err_str ="Unknown"; break;
 	    } 
-	    (void)fprintf(stderr, 
-			  "gpsctl: no gpsd running or network error: %s.\n", 
+	    gpsd_report(LOG_ERROR, "no gpsd running or network error: %s.\n", 
 			  err_str);
 	    lowlevel = true;
 	}
@@ -210,16 +243,16 @@ int main(int argc, char **argv)
 	assert(gpsdata != NULL);
 	(void)gps_query(gpsdata, "K\n");
 	if (gpsdata->ndevices == 0) {
-	    (void)fprintf(stderr, "gpsctl: no devices connected.\n"); 
+	    gpsd_report(LOG_ERROR, "no devices connected.\n"); 
 	    (void)gps_close(gpsdata);
 	    exit(1);
 	} else if (gpsdata->ndevices > 1 && device == NULL) {
-	    (void)fprintf(stderr, 
-			  "gpsctl: multiple devices and no device specified.\n");
+	    gpsd_report(LOG_ERROR, 
+			"multiple devices and no device specified.\n");
 	    (void)gps_close(gpsdata);
 	    exit(1);
 	}
-	gpsd_report(LOG_PROG, "gpsctl: %d device(s) found.\n", gpsdata->ndevices);
+	gpsd_report(LOG_PROG, "%d device(s) found.\n", gpsdata->ndevices);
 
 	if (gpsdata->ndevices > 1) {
 	    int i;
@@ -227,7 +260,7 @@ int main(int argc, char **argv)
 	    for (i = 0; i < gpsdata->ndevices; i++)
 		if (strcmp(device, gpsdata->devicelist[i]) == 0)
 		    goto foundit;
-	    (void)fprintf(stderr,  "gpsctl: specified device not found.\n");
+	    gpsd_report(LOG_ERROR, "specified device not found.\n");
 	    (void)gps_close(gpsdata);
 	    exit(1);
 	foundit:
@@ -235,47 +268,85 @@ int main(int argc, char **argv)
 	}
 
 	/* if no control operation was specified, just ID the device */
-	if (speed==NULL && !to_nmea && !to_binary && !reset) {
+	if (speed==NULL && rate == NULL && !to_nmea && !to_binary && !reset) {
 	    /* the O is to force a device binding */
 	    (void)gps_query(gpsdata, "OFIB");
-	    gpsd_report(LOG_SHOUT, "gpsctl: %s identified as %s at %d\n",
+	    gpsd_report(LOG_SHOUT, "%s identified as %s at %d\n",
 			gpsdata->gps_device,gpsdata->gps_id,gpsdata->baudrate);
 	    exit(0);
 	}
 
+	status = 0;
+#ifdef ALLOW_RECONFIGURE
 	if (reset)
 	{
-	    gpsd_report(LOG_PROG, "gpsctl: cannot reset with gpsd running.\n");
+	    gpsd_report(LOG_PROG, "cannot reset with gpsd running.\n");
 	    exit(0);
 	}
 
-	status = 0;
 	if (to_nmea) {
 	    (void)gps_query(gpsdata, "N=0");
 	    if (gpsdata->driver_mode != MODE_NMEA) {
-		(void)fprintf(stderr, "gpsctl: %s mode change to NMEA failed\n", gpsdata->gps_device);
+		gpsd_report(LOG_ERROR, "%s mode change to NMEA failed\n", gpsdata->gps_device);
 		status = 1;
 	    } else
-		gpsd_report(LOG_PROG, "gpsctl: %s mode change succeeded\n", gpsdata->gps_device);
+		gpsd_report(LOG_PROG, "%s mode change succeeded\n", gpsdata->gps_device);
 	}
 	else if (to_binary) {
 	    (void)gps_query(gpsdata, "N=1");
 	    if (gpsdata->driver_mode != MODE_BINARY) {
-		(void)fprintf(stderr, "gpsctl: %s mode change to native mode failed\n", gpsdata->gps_device);
+		gpsd_report(LOG_ERROR, "%s mode change to native mode failed\n", gpsdata->gps_device);
 		status = 1;
 	    } else
-		gpsd_report(LOG_PROG, "gpsctl: %s mode change succeeded\n", gpsdata->gps_device);
+		gpsd_report(LOG_PROG, "%s mode change succeeded\n", gpsdata->gps_device);
 	}
 	if (speed != NULL) {
-	    (void)gps_query(gpsdata, "B=%s", speed);
+	    char parity = 'N';
+	    char stopbits = '1';
+	    if (strchr(speed, ':')==NULL)
+		(void)gps_query(gpsdata, "B=%s", speed);
+	    else {
+		char *modespec = strchr(speed, ':');
+		/*@ +charint @*/
+		status = 0;
+		if (modespec!=NULL) {
+		    *modespec = '\0';
+		    if (strchr("78", *++modespec) == NULL) {
+			gpsd_report(LOG_ERROR, "No support for that word lengths.\n");
+			status = 1;
+		    }
+		    parity = *++modespec;
+		    if (strchr("NOE", parity) == NULL) {
+			gpsd_report(LOG_ERROR, "What parity is '%c'?\n", parity);
+			status = 1;
+		    }
+		    stopbits = *++modespec;
+		    if (strchr("12", stopbits) == NULL) {
+			gpsd_report(LOG_ERROR, "Stop bits must be 1 or 2.\n");
+			status = 1;
+		    }
+		}
+		if (status == 0)
+		    (void)gps_query(gpsdata, "B=%s 8 %c %c", 
+				    speed, parity, stopbits);
+	    }
 	    if (atoi(speed) != (int)gpsdata->baudrate) {
-		(void)fprintf( stderr, "gpsctl: %s speed change failed\n", gpsdata->gps_device);
+		gpsd_report(LOG_ERROR, "%s driver won't support %s%c%d\n", 
+			    gpsdata->gps_device,
+			    speed, parity, stopbits);
 		status = 1;
 	    } else
-		gpsd_report(LOG_PROG, "gpsctl: %s speed change succeeded\n", gpsdata->gps_device);
+		gpsd_report(LOG_PROG, "%s change to %s%c%d succeeded\n", 
+			    gpsdata->gps_device,
+			    speed, parity, stopbits);
 	}
+	if (rate != NULL) {
+	    (void)gps_query(gpsdata, "C=%\n", rate);
+	}
+#endif /* ALLOW_RECONFIGURE */
 	(void)gps_close(gpsdata);
 	exit(status);
+#ifdef ALLOW_RECONFIGURE
     } else if (reset) {
 	/* hard reset will go through lower-level operations */
 	const int speeds[] = {2400, 4800, 9600, 19200, 38400, 57600, 115200};
@@ -284,7 +355,7 @@ int main(int argc, char **argv)
 	int i;
 
 	if (device == NULL || forcetype == NULL) {
-		(void)fprintf(stderr,  "gpsctl: device and type must be specified for the reset operation.\n");
+		gpsd_report(LOG_ERROR, "device and type must be specified for the reset operation.\n");
 		exit(1);
 	    }
 
@@ -295,11 +366,11 @@ int main(int argc, char **argv)
 	session.device_type = forcetype;
 	(void)gpsd_open(&session);
 	(void)gpsd_set_raw(&session);
-	(void)session.device_type->speed_switcher(&session, 4800);
+	(void)session.device_type->speed_switcher(&session, 4800, 'N', 1);
 	(void)tcdrain(session.gpsdata.gps_fd);
 	for(i = 0; i < (int)(sizeof(speeds) / sizeof(speeds[0])); i++) {
 	    (void)gpsd_set_speed(&session, speeds[i], 'N', 1);
-	    (void)session.device_type->speed_switcher(&session, 4800);
+	    (void)session.device_type->speed_switcher(&session, 4800, 'N', 1);
 	    (void)tcdrain(session.gpsdata.gps_fd);
 	}
 	gpsd_set_speed(&session, 4800, 'N', 1);
@@ -309,6 +380,7 @@ int main(int argc, char **argv)
 	gpsd_wrap(&session);
 	exit(0);
 	/*@ +mustfreeonly +immediatetrans @*/
+#endif /* ALLOW_RECONFIGURE */
     } else {
 	/* access to the daemon failed, use the low-level facilities */
 	static struct gps_context_t	context;	/* start it zeroed */
@@ -316,6 +388,9 @@ int main(int argc, char **argv)
 	/*@ -mustfreeonly -immediatetrans @*/
 	session.context = &context;	/* in case gps_init isn't called */
 
+	if (echo)
+	    context.readonly = true;
+
 	(void) alarm(timeout);
 	(void) signal(SIGALRM, onsig);
 	/*
@@ -327,11 +402,11 @@ int main(int argc, char **argv)
 	    int seq;
 
 	    if (device == NULL) {
-		(void)fprintf(stderr,  "gpsctl: device must be specified for low-level access.\n");
+		gpsd_report(LOG_ERROR, "device must be specified for low-level access.\n");
 		exit(1);
 	    }
 	    gpsd_init(&session, &context, device);
-	    gpsd_report(LOG_PROG, "gpsctl: initialization passed.\n");
+	    gpsd_report(LOG_PROG, "initialization passed.\n");
 	    if (gpsd_activate(&session, false) == -1) {
 		gpsd_report(LOG_ERROR,
 			      "activation of device %s failed, errno=%d\n",
@@ -346,12 +421,12 @@ int main(int argc, char **argv)
 		    exit(2);
 		} else {
 		    gpsd_report(LOG_IO,
-				"autodectection after %d reads.\n", seq);
+				"autodetection after %d reads.\n", seq);
 		    (void) alarm(0);
 		    break;
 		}
 	    }
-	    gpsd_report(LOG_PROG, "gpsctl: %s looks like a %s at %d.\n",
+	    gpsd_report(LOG_PROG, "%s looks like a %s at %d.\n",
 			device, gpsd_id(&session), session.gpsdata.baudrate);
 
 	    if (forcetype!=NULL && strcmp("Generic NMEA", session.device_type->type_name) !=0 && strcmp(forcetype->type_name, session.device_type->type_name)!=0) {
@@ -372,7 +447,7 @@ int main(int argc, char **argv)
 			break;
 		}
 	    }
-	    gpsd_report(LOG_SHOUT, "gpsctl: %s identified as a %s at %d.\n",
+	    gpsd_report(LOG_SHOUT, "%s identified as a %s at %d.\n",
 			device, gpsd_id(&session), session.gpsdata.baudrate);
 	}
 
@@ -390,6 +465,7 @@ int main(int argc, char **argv)
 
 	/* now perform the actual control function */
 	status = 0;
+#ifdef ALLOW_RECOBFIGURE
 	/*@ -nullderef @*/
 	if (to_nmea || to_binary) {
 	    if (session.device_type->mode_switcher == NULL) {
@@ -417,7 +493,7 @@ int main(int argc, char **argv)
 		 * gets ignored or flushed.
 		 */
 		if (!echo) {
-		    /* suppresses probing for subrtpes */
+		    /* suppresses probing for subtypes */
 		    context.readonly = true;
 		    (void)sleep(1);
 		    (void) alarm(timeout);
@@ -425,52 +501,97 @@ int main(int argc, char **argv)
 			if (get_packet(&session) == ERROR_SET) {
 			    continue;
 			} else if (session.packet.type == target_type) {
-			    alarm(0);
+			    (void)alarm(0);
 			    break;
 			}
 		    }
 		    context.readonly = false;
 		}
+		/*@ -nullpass @*/
 		gpsd_report(LOG_SHOUT, "after mode change, %s looks like a %s at %d.\n",
 			    device, gpsd_id(&session), session.gpsdata.baudrate);
+		/*@ +nullpass @*/
 	    }
 	}
 	if (speed) {
-	    if (session.device_type->speed_switcher == NULL) {
-		gpsd_report(LOG_ERROR, 
-			    "%s devices have no speed switch.\n",
-			    session.device_type->type_name);
-		status = 1;
-	    }
-	    else if (!session.device_type->speed_switcher(&session, 
-							  (speed_t)atoi(speed))) {
-		(void)fprintf(stderr, "gpsctl: speed change failed.\n");
-		status = 1;
-	    }
-	}
-	/*@ -compdef @*/
-	if (control) {
-	    /* 
-	     * First, handle exceptional cases.  These are GPS chips
-	     * like the Zodiac and UBX in which the packet payload is 
-	     * not contiguous,with class or type fields before the data
-	     * length.  This means they don't fit well in the model 
-	     * assumed by the driver interface.
-	     */
-	    extern struct gps_type_t ubx_binary;
-	    /*@ -usedef @*/
-	    if (session.device_type == &ubx_binary) {
-		if (!ubx_write(session.gpsdata.gps_fd,
-			       (unsigned)cooked[0], (unsigned)cooked[1],
-			       (unsigned char *)cooked + 2, 
-			       (unsigned short)(cooklen - 2))) {
-		    gpsd_report(LOG_ERROR, "control transmission failed.\n");
+	    char parity = echo ? 'N': session.gpsdata.parity;
+	    int stopbits = echo ? 1 : session.gpsdata.stopbits;
+	    char *modespec;
+
+	    modespec = strchr(speed, ':');
+	    /*@ +charint @*/
+	    status = 0;
+	    if (modespec!=NULL) {
+		*modespec = '\0';
+		if (strchr("78", *++modespec) == NULL) {
+		    gpsd_report(LOG_ERROR, "No support for that word lengths.\n");
 		    status = 1;
 		}
-      
-	    } 
-	    /*@ +usedef @*/
-	    else if (session.device_type->control_send == NULL) {
+		parity = *++modespec;
+		if (strchr("NOE", parity) == NULL) {
+		    gpsd_report(LOG_ERROR, "What parity is '%c'?\n", parity);
+		    status = 1;
+		}
+		stopbits = *++modespec;
+		if (strchr("12", parity) == NULL) {
+		    gpsd_report(LOG_ERROR, "Stop bits must be 1 or 2.\n");
+		    status = 1;
+		}
+		stopbits = (int)(stopbits-'0');
+	    }
+	    if (status == 0) {
+		if (session.device_type->speed_switcher == NULL) {
+		    gpsd_report(LOG_ERROR, 
+				"%s devices have no speed switch.\n",
+				session.device_type->type_name);
+		    status = 1;
+		}
+		else if (session.device_type->speed_switcher(&session, 
+							     (speed_t)atoi(speed),
+							     parity, 
+							     stopbits)) {
+		    /*
+		     * See the comment attached to the 'B' command in gpsd.
+		     * Probably not needed here, but it can't hurt.
+		     */
+		    (void)tcdrain(session.gpsdata.gps_fd);
+		    (void)usleep(50000);
+		    gpsd_report(LOG_PROG, "%s change to %s%c%d succeeded\n", 
+			    session.gpsdata.gps_device,
+			    speed, parity, stopbits);
+		} else {
+		    gpsd_report(LOG_ERROR, "%s driver won't support %s%c%d.\n",
+				session.gpsdata.gps_device,
+				speed, parity, stopbits);
+		    status = 1;
+		}
+	    }
+	}
+	if (rate) {
+	    bool write_enable = context.readonly;
+	    context.readonly = false;
+	    if (session.device_type->rate_switcher == NULL) {
+		gpsd_report(LOG_ERROR, 
+			      "%s devices have no rate switcher.\n",
+			      session.device_type->type_name);
+		status = 1;
+	    } else {
+		double rate_dbl = strtod(rate, NULL);
+
+		if (!session.device_type->rate_switcher(&session, rate_dbl)) {
+		    gpsd_report(LOG_ERROR, "rate switch failed.\n");
+		    status = 1;
+		}
+	    }
+	    context.readonly = write_enable;
+	}
+#endif /* ALLOW_RECONFIGURE */
+#ifdef ALLOW_CONTROLSEND
+	/*@ -compdef @*/
+	if (control) {
+	    bool write_enable = context.readonly;
+	    context.readonly = false;
+	    if (session.device_type->control_send == NULL) {
 		gpsd_report(LOG_ERROR, 
 			      "%s devices have no control sender.\n",
 			      session.device_type->type_name);
@@ -483,9 +604,10 @@ int main(int argc, char **argv)
 		    status = 1;
 		}
 	    }
+	    context.readonly = write_enable;
 	}
 	/*@ +compdef @*/
-
+#endif /* ALLOW_CONTROLSEND */
 
 	if (forcetype == NULL || !echo) {
 	    /*
