@@ -1,4 +1,4 @@
-/* $Id: gpsd_dbus.c 4955 2009-01-08 15:14:50Z esr $ */
+/* $Id: gpsd_dbus.c 5277 2009-02-24 19:17:05Z esr $ */
 #include <sys/types.h>
 #include <stdio.h>
 #include "gpsd_config.h"
@@ -40,34 +40,8 @@ void send_dbus_fix(struct gps_device_t* channel) {
     gpsfix = &(gpsdata->fix);
     devname = gpsdata->gps_device;
 
-    message = dbus_message_new_signal( "/org/gpsd", "org.gpsd", "fix");
-
-    /* the old unnamed format - should be removed at some point */
-    dbus_message_append_args (message,
-			      DBUS_TYPE_DOUBLE, &(gpsfix->time),
-			      DBUS_TYPE_INT32,	&(gpsfix->mode),
-			      DBUS_TYPE_DOUBLE,	&(gpsfix->ept),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->latitude),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->longitude),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->eph),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->altitude),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->epv),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->track),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->epd),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->speed),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->eps),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->climb),
-			      DBUS_TYPE_DOUBLE, &(gpsfix->epc),
-			      DBUS_TYPE_INVALID);
-
-    dbus_message_set_no_reply(message, TRUE);
-
-    /* message is complete time to send it */
-    dbus_connection_send(connection, message, &serial);
-    dbus_message_unref(message);
-
     /* Send the named signel.  */
-    message = dbus_message_new_signal("/org/gpsd", "org.gpsd", "namedfix");
+    message = dbus_message_new_signal("/org/gpsd", "org.gpsd", "fix");
     dbus_message_append_args (message,
 			      DBUS_TYPE_DOUBLE, &(gpsfix->time),
 			      DBUS_TYPE_INT32,	&(gpsfix->mode),
