@@ -1,8 +1,8 @@
-/* $Id: gpsd_dbus.h 4543 2007-12-12 21:34:19Z esr $ */
-#ifdef DBUS_ENABLE
+/* $Id: gpsd_dbus.h 4794 2008-08-03 16:42:57Z ckuethe $ */
+#ifndef _GPSD_DBUS_H_
+#define _GPSD_DBUS_H_
 
-#ifndef _gpsd_dbus_h_
-#define _gpsd_dbus_h_
+#ifdef DBUS_ENABLE
 
 #include <dbus/dbus.h>
 
@@ -11,6 +11,6 @@
 int initialize_dbus_connection (void);
 void send_dbus_fix (struct gps_device_t* channel);
 
-#endif /* _gpsd_dbus_h_ */
-
 #endif
+
+#endif /* _GPSD_DBUS_H_ */
