@@ -1,4 +1,4 @@
-/* $Id: libgps.c 5092 2009-01-29 00:06:04Z esr $ */
+/* $Id: libgps.c 5498 2009-03-18 17:37:53Z esr $ */
 /* libgps.c -- client interface library for the gpsd daemon */
 #include <sys/time.h>
 #include <stdio.h>
@@ -11,127 +11,15 @@
 #include <pthread.h>	/* pacifies OpenBSD's compiler */
 #endif
 #include <math.h>
+#include <locale.h>
 
 #include "gpsd_config.h"
 #include "gpsd.h"
 
-#ifdef HAVE_SETLOCALE
-#include <locale.h>
-#endif
-
 #ifdef S_SPLINT_S
 extern char *strtok_r(char *, const char *, char **);
 #endif /* S_SPLINT_S */
 
-/* convert double degrees to a static string and return a pointer to it
- *
- * deg_str_type:
- *   	deg_dd     : return DD.dddddd
- *      deg_ddmm   : return DD MM.mmmm'
- *      deg_ddmmss : return DD MM' SS.sss"
- *
- */
-/*@observer@*/char *deg_to_str( enum deg_str_type type,  double f) 
-{
-    static char str[40];
-    int dsec, sec, deg, min;
-    long frac_deg;
-    double fdsec, fsec, fdeg, fmin;
-
-    if ( f < 0 || f > 360 ) {
-	(void)strlcpy( str, "nan", 40);
-	return str;
-    }
-
-    fmin = modf( f, &fdeg);
-    deg = (int)fdeg;
-    frac_deg = (long)(fmin * 1000000);
-
-    if ( deg_dd == type ) {
-	/* DD.dddddd */
-	(void)snprintf(str, sizeof(str), "%3d.%06ld", deg,frac_deg);
-	return str;
-    }
-    fsec = modf( fmin * 60, &fmin);
-    min = (int)fmin;
-    sec = (int)(fsec * 10000.0);
-
-    if ( deg_ddmm == type ) {
-	/* DD MM.mmmm */
-	(void)snprintf(str,sizeof(str), "%3d %02d.%04d'", deg,min,sec);
-	return str;
-    }
-    /* else DD MM SS.sss */
-    fdsec = modf( fsec * 60, &fsec);
-    sec = (int)fsec;
-    dsec = (int)(fdsec * 1000.0);
-    (void)snprintf(str,sizeof(str), "%3d %02d' %02d.%03d\"", deg,min,sec,dsec);
-
-    return str;
-}
-
-/* 
- * check the environment to determine proper GPS units
- *
- * clients should only call this if no user preference is specified on 
- * the command line or via X resources.
- *
- * return imperial    - Use miles/feet
- *        nautical    - Use knots/feet
- *        metric      - Use km/meters
- *        unspecified - use compiled default
- * 
- * In order check these environment vars:
- *    GPSD_UNITS one of: 
- *            	imperial   = miles/feet
- *              nautical   = knots/feet
- *              metric     = km/meters
- *    LC_MEASUREMENT
- *		en_US      = miles/feet
- *              C          = miles/feet
- *              POSIX      = miles/feet
- *              [other]    = km/meters
- *    LANG
- *		en_US      = miles/feet
- *              C          = miles/feet
- *              POSIX      = miles/feet
- *              [other]    = km/meters
- *
- * if none found then return compiled in default
- */
-enum unit gpsd_units(void)
-{
-	char *envu = NULL;
-
-#ifdef HAVE_SETLOCALE
-	(void)setlocale(LC_NUMERIC, "C");
-#endif
-  	if ((envu = getenv("GPSD_UNITS")) != NULL && *envu != '\0') {
-		if (0 == strcasecmp(envu, "imperial")) {
-			return imperial;
-		}
-		if (0 == strcasecmp(envu, "nautical")) {
-			return nautical;
-		}
-		if (0 == strcasecmp(envu, "metric")) {
-			return metric;
-		}
-		/* unrecognized, ignore it */
-	}
- 	if (((envu = getenv("LC_MEASUREMENT")) != NULL && *envu != '\0') 
- 	    || ((envu = getenv("LANG")) != NULL && *envu != '\0')) {
-		if (strcasecmp(envu, "en_US")==0 
-		    || strcasecmp(envu, "C")==0
-		    || strcasecmp(envu, "POSIX")==0) {
-			return imperial;
-		}
-		/* Other, must be metric */
-		return metric;
-	}
-	/* TODO: allow a compile time default here */
-	return unspecified;
-}
-
 struct gps_data_t *gps_open(const char *host, const char *port)
 /* open a connection to a gpsd daemon */
 {
@@ -184,20 +72,53 @@ void gps_set_raw_hook(struct gps_data_t *gpsdata,
     gpsdata->raw_hook = hook;
 }
 
-/*@ -branchstate -usereleased @*/
+/*@ -branchstate -usereleased -mustfreefresh @*/
 static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
-/* unpack a daemon response into a status structure */
+/* unpack a gpsd response into a status structure, buf must be writeable */
 {
     char *ns, *sp, *tp;
     int i;
 
+    /*
+     * Get the decimal separator for the current application locale.
+     * This looks thread-unsafe, but it's not.  The key is that
+     * character assignment is atomic.
+     */
+    static char decimal_point = '\0';
+    if (decimal_point == '\0') {
+	struct lconv *locale_data = localeconv();
+	if (locale_data != NULL && locale_data->decimal_point[0] != '.')
+	    decimal_point = locale_data->decimal_point[0];
+    }
+
     for (ns = buf; ns; ns = strstr(ns+1, "GPSD")) {
 	if (/*@i1@*/strncmp(ns, "GPSD", 4) == 0) {
+	    bool eol = false;
 	    /* the following should execute each time we have a good next sp */
 	    for (sp = ns + 5; *sp != '\0'; sp = tp+1) {
 		tp = sp + strcspn(sp, ",\r\n");
-		if (*tp == '\0') tp--;
-		else *tp = '\0';
+		eol = *tp == '\r' || *tp == '\n';
+		if (*tp == '\0')
+		    tp--;
+		else
+		    *tp = '\0';
+
+		/*
+		 * The daemon always emits the Anglo-American and SI
+		 * decimal point.  Hack these into whatever the 
+		 * application locale requires if it's not the same.
+		 * This has to happen *after* we grab the next 
+		 * comma-delimited response, or we'll lose horribly
+		 * in locales where the decimal separator is comma.
+		 */
+		if (decimal_point != '\0') {
+		    char *cp;
+		    for (cp = sp; cp < tp; cp++)
+			if (*cp == '.')
+			    *cp = decimal_point;
+		}
+
+		/* note, there's a bit of skip logic after the switch */
 
 		switch (*sp) {
 		case 'A':
@@ -285,13 +206,14 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			/*@ -nullderef -nullpass -mustfreeonly -dependenttrans @*/
 			char *rc = strdup(sp);
 			char *sp2 = rc;
+			char *ns2 = ns;
 			gpsdata->ndevices = (int)strtol(sp2+2, &sp2, 10);
 			gpsdata->devicelist = (char **)calloc(
 			    (size_t)gpsdata->ndevices,
 			    sizeof(char **));
 			/*@ -nullstate -mustfreefresh @*/
-			gpsdata->devicelist[i=0] = strdup(strtok_r(sp2+1, " \r\n", &ns));
-			while ((sp2 = strtok_r(NULL, " \r\n",  &ns)))
+			gpsdata->devicelist[i=0] = strdup(strtok_r(sp2+1, " \r\n", &ns2));
+			while ((sp2 = strtok_r(NULL, " \r\n",  &ns2)))
 			    gpsdata->devicelist[++i] = strdup(sp2);
 			free(rc);
 			/*@ +nullstate +mustfreefresh @*/
@@ -500,6 +422,14 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    /*@ -matchanyintegral +formatcode @*/
 		    break;
 		}
+
+		/* 
+		 * Skip to next GPSD when we see \r or \n; 
+		 * we don't want to try interpreting stuff
+		 * in between that might be raw mode data.
+		 */
+		if (eol)
+		    break;
 	    }
 	}
     }
@@ -511,7 +441,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 	gpsdata->thread_hook(gpsdata, buf, strlen(buf), 1);
 }
 /*@ +nullstate +compdef @*/
-/*@ -branchstate +usereleased @*/
+/*@ -branchstate +usereleased +mustfreefresh @*/
 
 /*
  * return: 0, success
@@ -617,6 +547,7 @@ int gps_del_callback(struct gps_data_t *gpsdata, pthread_t *handler)
 }
 #endif /* HAVE_LIBPTHREAD */
 
+
 #ifdef TESTMAIN
 /*
  * A simple command-line exerciser for the library.
@@ -679,15 +610,58 @@ static void dumpline(struct gps_data_t *ud UNUSED, char *buf,
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
+#include <getopt.h>
+#include <signal.h>
+
+static void onsig(int sig)
+{
+    (void)fprintf(stderr, "libgps: died with signal %d\n", sig);
+    exit(1);
+}
+
+/* must start zeroed, otherwise the unit test will try to chaser garbage pinter fields. */
+struct gps_data_t gpsdata;
+static char buf[] = "GPSD,O=RMC 1207318966.000 0.005 49.026225 12.188348 375.20 19.20 10.40 70.8900 24.899 0.000 75.6699 38.40 ? 3\r\n$GPVTG,70.89,T,,M,48.40,N,89.6,K,A*34\r\n";
+
+static void unpack_unit_test(void)
+/* torture the unpacking function */
+{
+
+    (void)signal(SIGSEGV, onsig);
+    (void)signal(SIGBUS, onsig);
+
+    gps_unpack(buf, &gpsdata);
+    data_dump(&gpsdata, time(NULL));
+}
 
 int main(int argc, char *argv[])
 {
     struct gps_data_t *collect;
     char buf[BUFSIZ];
+    int option;
+    bool unpack_test = false;
 
-    collect = gps_open(NULL, 0);
-    gps_set_raw_hook(collect, dumpline);
-    if (optind < argc) {
+    while ((option = getopt(argc, argv, "uh?")) != -1) {
+	switch (option) {
+	case 'u':
+	    unpack_test = true;
+	    break;
+	case '?':
+	case 'h':
+	default:
+	    (void)fputs("usage: libps [-u]\n", stderr);
+	    exit(1);
+	}
+    }
+
+    if (unpack_test) {
+	unpack_unit_test();
+	return 0;
+    } else if ((collect = gps_open(NULL, 0)) == NULL) {
+	(void)fputs("Daemon is not running.\n", stdout);
+	exit(1);
+    } else if (optind < argc) {
+	gps_set_raw_hook(collect, dumpline);
 	strlcpy(buf, argv[optind], BUFSIZ);
 	strlcat(buf,"\n", BUFSIZ);
 	gps_query(collect, buf);
@@ -695,6 +669,7 @@ int main(int argc, char *argv[])
     } else {
 	int	tty = isatty(0);
 
+	gps_set_raw_hook(collect, dumpline);
 	if (tty)
 	    (void)fputs("This is the gpsd exerciser.\n", stdout);
 	for (;;) {
